@@ -27,8 +27,11 @@ use Crypt::Perl::X ();
 #NOTE: This needs never to use Crypt::Perl::ECDSA::DB
 #so that extract_openssl_curves.pl will work.
 
-use constant OID_prime_field => '1.2.840.10045.1.1';
-use constant OID_characteristic_two_field => '1.2.840.10045.1.2';
+use constant {
+    OID_ecPublicKey => '1.2.840.10045.2.1',
+    OID_prime_field => '1.2.840.10045.1.1',
+    OID_characteristic_two_field => '1.2.840.10045.1.2',
+};
 
 use constant EXPORTABLE => qw( p a b n h gx gy );
 
@@ -103,7 +106,7 @@ sub normalize {
         $params = $parsed_or_der;
     }
     else {
-        die 'TODO';
+        die Crypt::Perl::X::create('Generic', 'TODO');
     }
 
     my $field_type = $params->{'fieldID'}{'fieldType'};
@@ -112,9 +115,10 @@ sub normalize {
             die Crypt::Perl::X::create('ECDSA::CharacteristicTwoUnsupported');
         }
 
-        die "Unknown field type OID: “$field_type”";
+        die Crypt::Perl::X::create('Generic', "Unknown field type OID: “$field_type”");
     }
 
+    #“seed” isn’t necessary here for calculations (… right??)
     my %curve = (
         p => $params->{'fieldID'}{'parameters'}{'prime-field'},
         a => $params->{'curve'}{'a'},
