@@ -3,6 +3,8 @@ package Crypt::Perl::ECDSA::KeyBase;
 use strict;
 use warnings;
 
+use Try::Tiny;
+
 use parent qw(
     Crypt::Perl::KeyBase
 );
@@ -18,11 +20,10 @@ use Crypt::Perl::ECDSA::EC::DB ();
 use Crypt::Perl::ECDSA::EC::Point ();
 use Crypt::Perl::ECDSA::ECParameters ();
 use Crypt::Perl::ECDSA::NIST ();
+use Crypt::Perl::ECDSA::EncodedPoint ();
 use Crypt::Perl::ECDSA::Utils ();
 use Crypt::Perl::X ();
 
-use constant OID_ecPublicKey => '1.2.840.10045.2.1';
-
 use constant ASN1_SIGNATURE => q<
     SEQUENCE {
         r   INTEGER,
@@ -96,29 +97,33 @@ sub verify_jwa {
 }
 
 sub to_der_with_curve_name {
-    my ($self) = @_;
+    my ($self, %params) = @_;
 
-    return $self->_get_asn1_parts($self->_named_curve_parameters());
+    if ($params{'seed'}) {
+        die Crypt::Perl::X::create('Generic', '“seed” is meaningless to a named-curve export.');
+    }
+
+    return $self->_get_asn1_parts($self->_named_curve_parameters(), %params);
 }
 
 sub to_der_with_explicit_curve {
-    my ($self) = @_;
+    my ($self, @params) = @_;
 
-    return $self->_get_asn1_parts($self->_explicit_curve_parameters());
+    return $self->_get_asn1_parts($self->_explicit_curve_parameters(@params), @params);
 }
 
 sub to_pem_with_curve_name {
-    my ($self) = @_;
+    my ($self, @params) = @_;
 
-    my $der = $self->to_der_with_curve_name();
+    my $der = $self->to_der_with_curve_name(@params);
 
     return Crypt::Format::der2pem($der, $self->_PEM_HEADER());
 }
 
 sub to_pem_with_explicit_curve {
-    my ($self) = @_;
+    my ($self, @params) = @_;
 
-    my $der = $self->to_der_with_explicit_curve();
+    my $der = $self->to_der_with_explicit_curve(@params);
 
     return Crypt::Format::der2pem($der, $self->_PEM_HEADER());
 }
@@ -138,7 +143,7 @@ sub get_curve_name {
 sub get_struct_for_public_jwk {
     my ($self) = @_;
 
-    my ($xb, $yb) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->{'public'}->as_bytes() );
+    my ($xb, $yb) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->_decompress_public_point() );
 
     Module::Load::load('MIME::Base64');
 
@@ -156,7 +161,9 @@ sub get_jwa_alg {
     my $name = $self->get_curve_name();
 
     my $getter_cr = __PACKAGE__->can("JWA_CURVE_ALG_$name") or do {
-        die sprintf( "“%s” knows of no JWA “alg” for the curve “%s”!", ref($self), $name);
+        my $err = sprintf( "“%s” knows of no JWA “alg” for the curve “%s”!", ref($self), $name);
+
+        die Crypt::Perl::X::create('Generic', $err);
     };
 
     return $getter_cr->();
@@ -164,13 +171,34 @@ sub get_jwa_alg {
 
 #----------------------------------------------------------------------
 
+sub _set_public {
+    my ($self, $pub_in) = @_;
+
+    $self->{'_public'} = Crypt::Perl::ECDSA::EncodedPoint->new($pub_in);
+
+    return;
+}
+
+sub _compress_public_point {
+    my ($self) = @_;
+
+    return $self->{'_public'}->get_compressed();
+}
+
+sub _decompress_public_point {
+    my ($self) = @_;
+
+    return $self->{'_public'}->get_uncompressed( $self->_curve() );
+}
+
 sub _get_jwk_digest_cr {
     my ($self) = @_;
 
     my $name = $self->get_curve_name();
 
     my $getter_cr = $self->can("JWA_DIGEST_$name") or do {
-        die sprintf( "“%s” knows of no digest to use for JWA with the curve “%s”!", ref($self), $name);
+        my $err = sprintf( "“%s” knows of no digest to use for JWA with the curve “%s”!", ref($self), $name);
+        die Crypt::Perl::X::create('Generic', $err);
     };
 
     Module::Load::load('Digest::SHA');
@@ -189,8 +217,8 @@ sub _get_jwk_curve_name {
 sub _verify {
     my ($self, $msg, $r, $s) = @_;
 
-    if ($r->bge(1) && $s->bge(1)) {
-        my ($x, $y) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->{'public'}->as_bytes() );
+    if ($r->is_positive() && $s->is_positive()) {
+        my ($x, $y) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->_decompress_public_point() );
         $_ = Crypt::Perl::BigInt->from_bytes($_) for ($x, $y);
 
         my $curve = $self->_get_curve_obj();
@@ -207,7 +235,7 @@ sub _verify {
 
         my $n = $self->_curve()->{'n'};
 
-        if ($r < $n && $s < $n) {
+        if ($r->blt($n) && $s->blt($n)) {
             my $c = $s->copy()->bmodinv($n);
 
             my $u1 = $e->copy()->bmul($c)->bmod($n);
@@ -233,12 +261,7 @@ sub _G {
 sub _pad_bytes_for_asn1 {
     my ($self, $bytes) = @_;
 
-    my $curve_hr = $self->_curve();
-    my $nbytes = length $curve_hr->{'p'}->as_bytes();
-
-    substr( $bytes, 0, 0 ) = ("\0" x ($nbytes - length $bytes));
-
-    return $bytes;
+    return Crypt::Perl::ECDSA::Utils::pad_bytes_for_asn1( $bytes, $self->_curve()->{'p'} );
 }
 
 sub _named_curve_parameters {
@@ -251,8 +274,10 @@ sub _named_curve_parameters {
     };
 }
 
+#The idea is to emulate what Convert::ASN1 gives us as the parse.
 sub _explicit_curve_parameters {
-    my ($self) = @_;
+    my ($self, %params) = @_;
+
     my $curve_hr = $self->_curve();
 
     my ($gx, $gy) = map { $_->as_bytes() } @{$curve_hr}{'gx', 'gy'};
@@ -266,10 +291,17 @@ sub _explicit_curve_parameters {
         b => $curve_hr->{'b'}->as_bytes(),
     );
 
-    if ($curve_hr->{'seed'}) {
+    if ($params{'seed'} && $curve_hr->{'seed'}) {
+
+        #We don’t care about the bit count. (Right?)
         $curve{'seed'} = $curve_hr->{'seed'}->as_bytes();
     }
 
+    my $base = "\x{04}$gx$gy";
+    if ( $params{'compressed'} ) {
+        $base = Crypt::Perl::ECDSA::Utils::compress_point($base);
+    }
+
     return {
         ecParameters => {
             version => 1,
@@ -280,7 +312,7 @@ sub _explicit_curve_parameters {
                 },
             },
             curve => \%curve,
-            base => "\x{04}$gx$gy",
+            base => $base,
             order => $curve_hr->{'n'},
             cofactor => $curve_hr->{'h'},
         },
@@ -288,19 +320,11 @@ sub _explicit_curve_parameters {
 }
 
 sub __to_der {
-    my ($self, $macro, $template, $data_hr) = @_;
+    my ($self, $macro, $template, $data_hr, %params) = @_;
 
-    my $curve_hr = $self->_curve();
+    my $pub_bin = $params{'compressed'} ? $self->_compress_public_point() : $self->_decompress_public_point();
 
-    my $nbytes = Crypt::Perl::Math::ceil( $curve_hr->{'n'} / 8 );
-
-    my ($pub_x, $pub_y) = Crypt::Perl::ECDSA::Utils::split_G_or_public( $self->{'public'}->as_bytes() );
-
-    for my $str ( $pub_x, $pub_y ) {
-        $str = $self->_pad_bytes_for_asn1($str);
-    }
-
-    local $data_hr->{'publicKey'} = "\x04$pub_x$pub_y";
+    local $data_hr->{'publicKey'} = $pub_bin;
 
     Module::Load::load('Crypt::Perl::ASN1');
     my $asn1 = Crypt::Perl::ASN1->new()->prepare($template);
@@ -319,6 +343,11 @@ sub _add_params {
     my ($self, $params_struct) = @_;
 
     if (my $params = $params_struct->{'ecParameters'}) {
+
+        #Convert::ASN1 returns bit strings as an array of [ $content, $length ].
+        #Since normalize() wants that, we local() it here.
+        #local $params->{'curve'}{'seed'} = [$params->{'curve'}{'seed'}] if $params->{'curve'} && $params->{'curve'}{'seed'};
+
         $self->{'curve'} = Crypt::Perl::ECDSA::ECParameters::normalize($params);
     }
     else {
@@ -337,7 +366,7 @@ sub _curve_params_for_OID {
 sub _curve {
     my ($self) = @_;
 
-    return $self->{'curve'} ||= $self->_curve_params_for_OID();
+    return $self->{'curve'};
 }
 
 #----------------------------------------------------------------------
